/*
 * This file and its contents are supplied under the terms of the
 * Common Development and Distribution License ("CDDL"), version 1.0.
 * You may only use this file in accordance with the terms of version
 * 1.0 of the CDDL.
 *
 * A full copy of the text of the CDDL should have accompanied this
 * source.  A copy of the CDDL is also available via the Internet at
 * http://www.illumos.org/license/CDDL.
 */

/*
 * Copyright (c) 2014 Joyent, Inc.  All rights reserved.
 */

#pragma D depends_on library ip.d

inline int ETHERTYPE_PUP = (0x0200); 
inline int ETHERTYPE_802_MIN = (0x0600);
inline int ETHERTYPE_IP = (0x0800); 
inline int ETHERTYPE_ARP = (0x0806); 
inline int ETHERTYPE_REVARP = (0x8035);
inline int ETHERTYPE_AT = (0x809b); 
inline int ETHERTYPE_AARP = (0x80f3);
inline int ETHERTYPE_VLAN = (0x8100);
inline int ETHERTYPE_IPV6 = (0x86dd);
inline int ETHERTYPE_SLOW = (0x8809);
inline int ETHERTYPE_PPPOED = (0x8863);
inline int ETHERTYPE_PPPOES = (0x8864);
inline int ETHERTYPE_EAPOL = (0x888e);
inline int ETHERTYPE_RSN_PREAUTH = (0x88c7);
inline int ETHERTYPE_TRILL = (0x88c8);
inline int ETHERTYPE_FCOE = (0x8906);
inline int ETHERTYPE_MAX = (0xffff);


typedef struct etherinfo {
	uint8_t eth_dst[6];		/* Destination MAC addr */
	uint8_t eth_src[6];		/* Source MAC addr */
	uint16_t eth_type;		/* Ethertype */
	boolean_t eth_istagged;		/* Is the VLAN tag present */
	uint8_t eth_priority;		/* Priority tag */
	uint8_t eth_dei;		/* drop eligible indicator */
	uint16_t eth_vlanid;		/* VLAN ID */
	uintptr_t eth_header;		/* Pointer to start of header */
	uintptr_t eth_mblk;		/* Pointer to the mblk containing header */
} etherinfo_t;

#pragma D binding "1.12.1" translator
translator etherinfo_t < mblk_t *mp > {
	eth_dst = mp->b_rptr;
	eth_src = mp->b_rptr + 6;
	eth_type = ntohs(*(uint16_t *)(mp->b_rptr + 12)) == ETHERTYPE_VLAN ?
	    ntohs(*(uint16_t *)(mp->b_rptr + 16)) :
	    ntohs(*(uint16_t *)(mp->b_rptr + 12));
	eth_istagged = ntohs(*(uint16_t *)(mp->b_rptr + 12)) == ETHERTYPE_VLAN ?
	    1 : 0;
	eth_priority = ntohs(*(uint16_t *)(mp->b_rptr + 12)) == ETHERTYPE_VLAN ?
	    ntohs(*(uint16_t *)(mp->b_rptr + 14)) & 0xe000: 0;
	eth_dei = ntohs(*(uint16_t *)(mp->b_rptr + 12)) == ETHERTYPE_VLAN ?
	    ntohs(*(uint16_t *)(mp->b_rptr + 14)) & 0x1000: 0;
	eth_vlanid = ntohs(*(uint16_t *)(mp->b_rptr + 12)) == ETHERTYPE_VLAN ?
	    ntohs(*(uint16_t *)(mp->b_rptr + 14)) & 0x0fff: 0;
	eth_header = (uintptr_t)mp->b_rptr;
	eth_mblk = (uintptr_t)mp;
};
